//	window.h - The main window for the application.
//	----------------------------------------------------------------------------
//	This file is part of Mouse To OSC, a simple OSC-outputting program.
//	Copyright (c) 2006 Niall Moody.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//	----------------------------------------------------------------------------

#ifndef WINDOW_H_
#define WINDOW_H_

#include "PadControl.h"
#include "SliderControl.h"
#include "UDPSocket.h"
#include "OSCBundle.h"

//------------------------------------------------------------------------------
///	Structure containing the various parameters in a single slot.
typedef struct sValues
{
	///	Constructor.
	sValues()
	{
		int i;

		for(i=0;i<3;++i)
		{
			padValues[i].xAddress = T("");
			padValues[i].yAddress = T("");
			padValues[i].xMax = T("1.0");
			padValues[i].yMax = T("1.0");
			padValues[i].xMin = T("0.0");
			padValues[i].yMin = T("0.0");
			padValues[i].xVal = 0.0f;
			padValues[i].yVal = 0.0f;
			padValues[i].xCc = T("<CC>");
			padValues[i].yCc = T("<CC>");
			topSliderValues[i].address = T("");
			topSliderValues[i].max = T("1.0");
			topSliderValues[i].min = T("0.0");
			topSliderValues[i].val = 0.0f;
			topSliderValues[i].cc = T("<MIDI CC>");
			bottomSliderValues[i].address = T("");
			bottomSliderValues[i].max = T("1.0");
			bottomSliderValues[i].min = T("0.0");
			bottomSliderValues[i].val = 0.0f;
			bottomSliderValues[i].cc = T("<MIDI CC>");
		}
	}

	PadControlValues padValues[3];
	SliderControlValues topSliderValues[3];
	SliderControlValues bottomSliderValues[3];
} SlotValues;

//------------------------------------------------------------------------------
///	The main window stuff for the application.
class WindowContents : public Component,
					   public ChangeListener,
					   public MenuBarModel,
					   public ButtonListener,
					   public MidiInputCallback,
					   public ComboBoxListener
{
  public:
	///	Constructor.
	WindowContents();
	///	Destructor.
	~WindowContents();

	///	Used to place the various controls.
	void resized();

	///	Used to draw some lines separating the various sections.
	void paint(Graphics& g);

	///	Called whenever the user drags a slider etc.
	void changeListenerCallback(void *objectThatHasChanged);
	///	Called whenever the user clicks a slot button.
	void buttonClicked(Button *button);
	///	Called when the MIDI input is changed.
	void comboBoxChanged(ComboBox *comboBoxThatHasChanged);

	///	Returns the names of the menus.
	const StringArray getMenuBarNames(MenuBarComponent *menuBar);
	///	Returns a PopupMenu for the required menu.
	const PopupMenu getMenuForIndex(MenuBarComponent *menuBar,
									int topLevelMenuIndex,
									const String &menuName);
	///	Called when a menu item has been clicked.
	void menuItemSelected(MenuBarComponent *menuBar,
						  int menuItemID,
						  int topLevelMenuIndex);

	///	So we receive MIDI input messages.
	void handleIncomingMidiMessage(MidiInput *source,
								   const MidiMessage &message);

    juce_UseDebuggingNewOperator
  private:
	///	Initialises all the widgets.
	void fileNew();
	///	Loads widget values from a file.
	void fileOpen();
	///	Saves all the current slot values to a file.
	void fileSave();
	///	Saves all the current slot values to a file.
	void fileSaveAs();

	///	Fills a SlotValues struct from the current widget values.
	void fillSlotValues(SlotValues& slot);
	///	Updates the widgets from a SlotValues struct.
	void updateWidgets(const SlotValues& slot);

	///	Label for the address editor.
	Label *addressLabel;
	///	Text editor to set the address to send OSC messages to.
	TextEditor *addressEditor;
	///	Label for the port editor.
	Label *portLabel;
	///	Text editor to set the port to send OSC messages to.
	TextEditor *portEditor;
	///	Label for the MIDI input combo box.
	Label *midiInputLabel;
	///	Combo box to set the MIDI input device.
	ComboBox *midiInput;

	///	The 3 PadControls.
	PadControl **pads;
	///	The 1st 3 SliderControls.
	SliderControl **topSliders;
	///	The 2nd 3 SliderControls.
	SliderControl **bottomSliders;

	///	The 8 'slot' buttons.
	TextButton **slotButtons;
	///	The last selected slot button.
	int lastSlot;
	///	All the values for our current 8 slots.
	SlotValues slotValues[8];

	///	The path to the last saved file.
	String lastFile;
	///	The parent directory the PhilChooser was last in.
	File lastDir;

	///	The socket we're sending our data through.
	UDPSocket sock;
	///	Bundle sent when the user drags an xy pad.
	OSC::Bundle bundle;

	///	The current MIDI input device.
	MidiInput *midiInputDevice;

	///	Enum containing menu IDs.
	enum
	{
		FileNew = 1,
		FileOpen,
		FileSave,
		FileSaveAs,
		FileQuit,
		HelpAbout,
		SlotID
	};
};

#endif
